@@ -1,6 +1,6 @@
 #' Construct the random subspace ensemble classifier.
 #'
-#' \code{Rase} is a novel model-free ensemble classification framework to solve the sparse classification problem. In RaSE algorithm, for each of the B1 weak learners, B2 random subspaces are generated and the optimal one is chosen to train the model on the basis of some criterion.
+#' \code{RaSE} is a general ensemble classification framework to solve the sparse classification problem. In RaSE algorithm, for each of the B1 weak learners, B2 random subspaces are generated and the optimal one is chosen to train the model on the basis of some criterion.
 #' @export
 #' @importFrom MASS lda
 #' @importFrom MASS qda
@@ -10,6 +10,7 @@
 #' @importFrom caret knn3
 #' @importFrom caret createFolds
 #' @importFrom caret findLinearCombos
+#' @importFrom caret knnreg
 #' @importFrom doParallel registerDoParallel
 #' @importFrom doParallel stopImplicitCluster
 #' @importFrom foreach foreach
@@ -30,6 +31,10 @@
 #' @importFrom stats cov
 #' @importFrom stats cor
 #' @importFrom stats var
+#' @importFrom stats rnorm
+#' @importFrom stats rt
+#' @importFrom stats runif
+#' @importFrom stats deviance
 #' @importFrom ggplot2 ggplot
 #' @importFrom ggplot2 aes
 #' @importFrom ggplot2 aes_string
@@ -42,13 +47,14 @@
 #' @importFrom gridExtra grid.arrange
 #' @importFrom formatR tidy_eval
 #' @importFrom FNN KL.divergence
+#' @importFrom ranger ranger
 #' @param xtrain n * p observation matrix. n observations, p features.
 #' @param ytrain n 0/1 observatons.
 #' @param xval observation matrix for validation. Default = \code{NULL}. Useful only when \code{criterion} = 'validation'.
 #' @param yval 0/1 observation for validation. Default = \code{NULL}. Useful only when \code{criterion} = 'validation'.
 #' @param B1 the number of weak learners. Default = 200.
 #' @param B2 the number of subspace candidates generated for each weak learner. Default = 500.
-#' @param D the maximal subspace size when generating random subspaces from the uniform distribution. Default = \code{NULL}, which is \eqn{min(\sqrt n0, \sqrt n1, p)} when \code{base} = 'lda' and is \eqn{min(\sqrt n, p)} otherwise.
+#' @param D the maximal subspace size when generating random subspaces. Default = \code{NULL}, which is \eqn{min(\sqrt n0, \sqrt n1, p)} when \code{base} = 'qda' and is \eqn{min(\sqrt n, p)} otherwise.
 #' @param dist the distribution for features when generating random subspaces. Default = \code{NULL}, which represents the uniform distribution. First generate an integer \eqn{d} from \eqn{1,...,D} uniformly, then uniformly generate a subset with cardinality \eqn{d}.
 #' @param base the type of base classifier. Default = 'lda'.
 #' \itemize{
@@ -64,25 +70,34 @@
 #' @param criterion the criterion to choose the best subspace for each weak learner. Default = 'ric' when \code{base} = 'lda', 'qda', 'gamma'; default = 'ebic' and set \code{gam} = 0 when \code{base} = 'logistic'; default = 'loo' when \code{base} = 'knn'; default = 'training' when \code{base} = 'tree', 'svm', 'randomforest'.
 #' \itemize{
 #' \item ric: minimizing ratio information criterion with parametric estimation (Tian, Y. and Feng, Y., 2020). Available when \code{base} = 'lda', 'qda', 'gamma' or 'logistic'.
-#' \item nric: minimizing ratio information criterion with non-parametric estimation (Tian, Y. and Feng, Y., 2020; Wang, Q., Kulkarni, S.R. and Verdú, S., 2009). Available when \code{base} = 'lda', 'qda', 'gamma' or 'logistic'.
+#' \item nric: minimizing ratio information criterion with non-parametric estimation (Tian, Y. and Feng, Y., 2020). Available when \code{base} = 'lda', 'qda', 'gamma' or 'logistic'.
 #' \item training: minimizing training error. Not available when \code{base} = 'knn'.
 #' \item loo: minimizing leave-one-out error. Only available when  \code{base} = 'knn'.
 #' \item validation: minimizing validation error based on the validation data. Available for all base classifiers.
-#' \item cv: minimizing k-fold cross-validation error. k equals to the value of \code{cv}. Default = 10. Not available when \code{base} = 'gamma'.
-#' \item ebic: minimizing extended Bayesian information criterion (Chen, J. and Chen, Z., 2008; 2012). Need to assign value for \code{gam}. When \code{gam} = 0, it denotes the classical BIC. Available when \code{base} = 'lda', 'qda' or 'logistic'.
+#' \item cv: minimizing k-fold cross-validation error. k equals to the value of \code{cv}. Default = 5. Not available when \code{base} = 'gamma'.
+#' \item aic: minimizing Akaike information criterion (Akaike, H., 1973). Available when \code{base} = 'lda' or 'logistic'.
+#'
+#' AIC = -2 * log-likelihood + |S| * 2.
+#'
+#' \item bic: minimizing Bayesian information criterion (Schwarz, G., 1978). Available when \code{base} = 'lda' or 'logistic'.
+#'
+#' BIC = -2 * log-likelihood + |S| * log(n).
+#'
+#' \item ebic: minimizing extended Bayesian information criterion (Chen, J. and Chen, Z., 2008; 2012). Need to assign value for \code{gam}. When \code{gam} = 0, it denotes the classical BIC. Available when \code{base} = 'lda' or 'logistic'.
 #'
 #' EBIC = -2 * log-likelihood + |S| * log(n) + 2 * |S| * gam * log(p).
+#'
 #' }
 #' @param ranking whether the function outputs the selected percentage of each feature in B1 subspaces. Logistic, default = TRUE.
-#' @param k the number of nearest neightbors considered when \code{base} = 'knn'. Only useful when \code{base} = 'knn'.
+#' @param k the number of nearest neightbors considered when \code{base} = 'knn'. Only useful when \code{base} = 'knn'. Default = (3, 5, 7, 9, 11).
 #' @param cores the number of cores used for parallel computing. Default = 1.
 #' @param seed the random seed assigned at the start of the algorithm, which can be a real number or \code{NULL}. Default = \code{NULL}, in which case no random seed will be set.
 #' @param iteration the number of iterations. Default = 0.
 #' @param cutoff whether to use the empirically optimal threshold. Logistic, default = TRUE. If it is FALSE, the threshold will be set as 0.5.
-#' @param cv the number of cross-validations used. Default = 10. Only useful when \code{criterion} = 'cv'.
+#' @param cv the number of cross-validations used. Default = 5. Only useful when \code{criterion} = 'cv'.
 #' @param scale whether to normalize the data. Logistic, default = FALSE.
-#' @param C0 the threshold used to adjust the sampling probabilities of features when \code{iteration} > 0. Default = 0.1.
-#' @param kl.k the number of nearest neighbors used to estimate KL divergences when \code{criterion} = 'nric'. 2-dimensional vector. Default = \code{NULL}, in which case it will be set as \eqn{\sqrt n0, \sqrt n1}.
+#' @param C0 a positive constant used when \code{iteration} > 1. See Tian, Y. and Feng, Y., 2020 for details.
+#' @param kl.k the number of nearest neighbors used to estimate RIC in a non-parametric way. Default = \code{NULL}, which means that \eqn{k0 = floor(\sqrt n0)} and \eqn{k1 = floor(\sqrt n1)}. See Tian, Y. and Feng, Y., 2020 for details. Only available when \code{criterion} = 'nric'.
 #' @param ... additional arguments.
 #' @return An object with S3 class \code{'RaSE'}.
 #' \item{marginal}{the marginal probability for each class.}
@@ -90,27 +105,29 @@
 #' \item{criterion}{the criterion to choose the best subspace for each weak learner.}
 #' \item{B1}{the number of weak learners.}
 #' \item{B2}{the number of subspace candidates generated for each weak learner.}
+#' \item{D}{the maximal subspace size when generating random subspaces.}
 #' \item{iteration}{the number of iterations.}
 #' \item{fit.list}{sequence of B1 fitted base classifiers.}
 #' \item{cutoff}{the empirically optimal threshold.}
 #' \item{subspace}{sequence of subspaces correponding to B1 weak learners.}
 #' \item{ranking}{the selected percentage of each feature in B1 subspaces.}
 #' \item{scale}{a list of scaling parameters, including the scaling center and the scale parameter for each feature. Equals to \code{NULL} when the data is not scaled in \code{RaSE} model fitting.}
-#' \item{C0}{the threshold used to adjust the sampling probabilities of features when \code{iteration} > 0.}
-#' @seealso \code{\link{predict.RaSE}}, \code{\link{RaModel}}, \code{\link{print.RaSE}}, \code{\link{RaPlot}}.
+#' @seealso \code{\link{predict.RaSE}}, \code{\link{RaModel}}, \code{\link{print.RaSE}}, \code{\link{RaPlot}}, \code{\link{RaScreen}}.
 #' @references
-#' Tian, Y. and Feng, Y., 2020. RaSE: Random subspace ensemble classification. arXiv preprint arXiv:2006.08855.
+#' Tian, Y. and Feng, Y., 2021. RaSE: Random subspace ensemble classification. Journal of Machine Learning Research, 22, to appear.
 #'
 #' Chen, J. and Chen, Z., 2008. Extended Bayesian information criteria for model selection with large model spaces. Biometrika, 95(3), pp.759-771.
 #'
 #' Chen, J. and Chen, Z., 2012. Extended BIC for small-n-large-P sparse GLM. Statistica Sinica, pp.555-574.
 #'
-#' Wang, Q., Kulkarni, S.R. and Verdú, S., 2009. Divergence estimation for multidimensional densities via $ k $-nearest-neighbor distances. IEEE Transactions on Information Theory, 55(5), pp.2392-2405.#' @examples
+#' Akaike, H., 1973. Information theory and an extension of the maximum likelihood principle. In 2nd International Symposium on Information Theory, 1973 (pp. 267-281). Akademiai Kaido.
+#'
+#' Schwarz, G., 1978. Estimating the dimension of a model. The annals of statistics, 6(2), pp.461-464.
 #'
 #' @examples
 #' set.seed(0, kind = "L'Ecuyer-CMRG")
-#' train.data <- RaModel(1, n = 100, p = 50)
-#' test.data <- RaModel(1, n = 100, p = 50)
+#' train.data <- RaModel("classification", 1, n = 100, p = 50)
+#' test.data <- RaModel("classification", 1, n = 100, p = 50)
 #' xtrain <- train.data$x
 #' ytrain <- train.data$y
 #' xtest <- test.data$x
@@ -154,14 +171,19 @@
 #' }
 
 Rase <- function(xtrain, ytrain, xval = NULL, yval = NULL, B1 = 200, B2 = 500, D = NULL, dist = NULL, base = c("lda",
-    "qda", "knn", "logistic", "tree", "svm", "randomforest", "gamma"), criterion = NULL, ranking = TRUE, k = c(3, 5, 7, 9, 11), cores = 1,
-    seed = NULL, iteration = 0, cutoff = TRUE, cv = 10, scale = FALSE, C0 = 0.1, kl.k = NULL, ...) {
-
+    "qda", "knn", "logistic", "tree", "svm", "randomforest", "gamma", "NULL"), criterion = NULL, ranking = TRUE, k = c(3, 5, 7, 9, 11), cores = 1,
+    seed = NULL, iteration = 0, cutoff = TRUE, cv = 5, scale = FALSE, C0 = 0.1, kl.k = NULL, ...) {
 
     if (!is.null(seed)) {
         set.seed(seed, kind = "L'Ecuyer-CMRG")
     }
 
+    xtrain <- as.matrix(xtrain)
+    base <- match.arg(base)
+    p <- ncol(xtrain)
+    n <- length(ytrain)
+
+
     if (is.null(criterion)) {
         if (base == "lda" || base == "qda" || base == "gamma") {
             criterion <- "ric"
@@ -176,14 +198,9 @@ Rase <- function(xtrain, ytrain, xval = NULL, yval = NULL, B1 = 200, B2 = 500, D
     }
 
 
-    xtrain <- as.matrix(xtrain)
-    base <- match.arg(base)
-    p <- ncol(xtrain)
-    n <- length(ytrain)
     n0 <- sum(ytrain == 0)
     n1 <- sum(ytrain == 1)
 
-
     if(is.null(kl.k)) {
         kl.k <- floor(c(sqrt(n0), sqrt(n1)))
     }
@@ -242,7 +259,11 @@ Rase <- function(xtrain, ytrain, xval = NULL, yval = NULL, B1 = 200, B2 = 500, D
                   criterion = criterion, cv = cv, mu0.mle = mu0.mle, mu1.mle = mu1.mle, Sigma.mle = Sigma.mle, kl.k = kl.k, ...)
             }
 
-            subspace <- output[, 3]
+            if (is.matrix(output)) {
+                subspace <- output[, 3]
+            } else {
+                subspace <- output[3]
+            }
 
             s <- rep(0, p)
             for (i in 1:length(subspace)) {
@@ -250,15 +271,19 @@ Rase <- function(xtrain, ytrain, xval = NULL, yval = NULL, B1 = 200, B2 = 500, D
             }
 
 
-            dist <- s/sum(s)
+            dist <- s/B1
             dist[dist < C0/log(p)] <- C0/p
             dist[delete.ind] <- 0
 
-            ytrain.pred <- data.frame(matrix(unlist(output[, 2]), ncol = B1))
-
         }
 
-        fit.list <- output[, 1]
+        if (is.matrix(output)) {
+            ytrain.pred <- data.frame(matrix(unlist(output[, 2]), ncol = B1))
+            fit.list <- output[, 1]
+        } else {
+            ytrain.pred <- data.frame(matrix(unlist(output[2]), ncol = B1))
+            fit.list <- output[1]
+        }
 
     }
 
@@ -294,6 +319,8 @@ Rase <- function(xtrain, ytrain, xval = NULL, yval = NULL, B1 = 200, B2 = 500, D
         dist <- rep(1, p)
         dist[delete.ind] <- 0
 
+        # select.v <- NULL
+
         for (t in 1:(iteration + 1)) {
             output <- foreach(i = 1:B1, .combine = "rbind", .packages = "MASS") %dopar% {
                 S <- sapply(1:B2, function(j) {
@@ -316,23 +343,33 @@ Rase <- function(xtrain, ytrain, xval = NULL, yval = NULL, B1 = 200, B2 = 500, D
                   ...)
             }
 
-            subspace <- output[, 3]
+            if (is.matrix(output)) {
+                subspace <- output[, 3]
+            } else {
+                subspace <- output[3]
+            }
 
             s <- rep(0, p)
             for (i in 1:length(subspace)) {
                 s[subspace[[i]]] <- s[subspace[[i]]] + 1
             }
 
-            dist <- s/sum(s)
+            dist <- s/B1
             dist[dist < C0/log(p)] <- C0/p
             dist[delete.ind] <- 0
 
-            ytrain.pred <- data.frame(matrix(unlist(output[, 2]), ncol = B1))
+            # ytrain.pred <- data.frame(matrix(unlist(output[, 2]), ncol = B1))
 
         }
 
-        fit.list <- output[, 1]
-
+        # fit.list <- output[, 1]
+        if (is.matrix(output)) {
+            ytrain.pred <- data.frame(matrix(unlist(output[, 2]), ncol = B1))
+            fit.list <- output[, 1]
+        } else {
+            ytrain.pred <- data.frame(matrix(unlist(output[2]), ncol = B1))
+            fit.list <- output[1]
+        }
     }
 
     if (base == "knn") {
@@ -354,19 +391,29 @@ Rase <- function(xtrain, ytrain, xval = NULL, yval = NULL, B1 = 200, B2 = 500, D
                   criterion = criterion, cv = cv, ...)
             }
 
-            subspace <- output[, 3]
+            if (is.matrix(output)) {
+                subspace <- output[, 3]
+            } else {
+                subspace <- output[3]
+            }
 
             s <- rep(0, p)
             for (i in 1:length(subspace)) {
                 s[subspace[[i]]] <- s[subspace[[i]]] + 1
             }
 
-            dist <- s/sum(s)
+            # dist <- s/sum(s)
+            dist <- s/B1
             dist[dist < C0/log(p)] <- C0/p
-            ytrain.pred <- data.frame(matrix(unlist(output[, 2]), ncol = B1))
         }
 
-        fit.list <- output[, 1]
+        if (is.matrix(output)) {
+            ytrain.pred <- data.frame(matrix(unlist(output[, 2]), ncol = B1))
+            fit.list <- output[, 1]
+        } else {
+            ytrain.pred <- data.frame(matrix(unlist(output[2]), ncol = B1))
+            fit.list <- output[1]
+        }
     }
 
     if (base == "tree") {
@@ -388,19 +435,28 @@ Rase <- function(xtrain, ytrain, xval = NULL, yval = NULL, B1 = 200, B2 = 500, D
                   criterion = criterion, cv = cv, ...)
             }
 
-            subspace <- output[, 3]
+            if (is.matrix(output)) {
+                subspace <- output[, 3]
+            } else {
+                subspace <- output[3]
+            }
 
             s <- rep(0, p)
             for (i in 1:length(subspace)) {
                 s[subspace[[i]]] <- s[subspace[[i]]] + 1
             }
 
-            dist <- s/sum(s)
+            dist <- s/B1
             dist[dist < C0/log(p)] <- C0/p
-            ytrain.pred <- data.frame(matrix(unlist(output[, 2]), ncol = B1))
         }
 
-        fit.list <- output[, 1]
+        if (is.matrix(output)) {
+            ytrain.pred <- data.frame(matrix(unlist(output[, 2]), ncol = B1))
+            fit.list <- output[, 1]
+        } else {
+            ytrain.pred <- data.frame(matrix(unlist(output[2]), ncol = B1))
+            fit.list <- output[1]
+        }
     }
 
     if (base == "logistic" || base == "svm" || base == "randomforest") {
@@ -422,19 +478,28 @@ Rase <- function(xtrain, ytrain, xval = NULL, yval = NULL, B1 = 200, B2 = 500, D
                   criterion = criterion, cv = cv, ...)
             }
 
-            subspace <- output[, 3]
+            if (is.matrix(output)) {
+                subspace <- output[, 3]
+            } else {
+                subspace <- output[3]
+            }
 
             s <- rep(0, p)
             for (i in 1:length(subspace)) {
                 s[subspace[[i]]] <- s[subspace[[i]]] + 1
             }
 
-            dist <- s/sum(s)
+            dist <- s/B1
             dist[dist < C0/log(p)] <- C0/p
-            ytrain.pred <- data.frame(matrix(unlist(output[, 2]), ncol = B1))
         }
 
-        fit.list <- output[, 1]
+        if (is.matrix(output)) {
+            ytrain.pred <- data.frame(matrix(unlist(output[, 2]), ncol = B1))
+            fit.list <- output[, 1]
+        } else {
+            ytrain.pred <- data.frame(matrix(unlist(output[2]), ncol = B1))
+            fit.list <- output[1]
+        }
     }
 
 
@@ -473,19 +538,28 @@ Rase <- function(xtrain, ytrain, xval = NULL, yval = NULL, B1 = 200, B2 = 500, D
                   criterion = criterion, cv = cv, t0.mle = t0.mle, t1.mle = t1.mle, kl.k = kl.k, ...)
             }
 
-            subspace <- output[, 3]
+            if (is.matrix(output)) {
+                subspace <- output[, 3]
+            } else {
+                subspace <- output[3]
+            }
 
             s <- rep(0, p)
             for (i in 1:length(subspace)) {
                 s[subspace[[i]]] <- s[subspace[[i]]] + 1
             }
 
-            dist <- s/sum(s)
+            dist <- s/B1
             dist[dist < C0/log(p)] <- C0/p
-            ytrain.pred <- data.frame(matrix(unlist(output[, 2]), ncol = B1))
         }
 
-        fit.list <- output[, 1]
+        if (is.matrix(output)) {
+            ytrain.pred <- data.frame(matrix(unlist(output[, 2]), ncol = B1))
+            fit.list <- output[, 1]
+        } else {
+            ytrain.pred <- data.frame(matrix(unlist(output[2]), ncol = B1))
+            fit.list <- output[1]
+        }
     }
 
 
@@ -513,9 +587,12 @@ Rase <- function(xtrain, ytrain, xval = NULL, yval = NULL, B1 = 200, B2 = 500, D
     }
 
     stopImplicitCluster()
-    obj <- list(marginal = c(`class 0` = p0, `class 1` = 1 - p0), base = base, criterion = criterion, B1 = B1, B2 = B2,
-                iteration = iteration, fit.list = fit.list, cutoff = cutoff, subspace = subspace, ranking = rk, scale = scale.parameters, C0 = C0)
+    obj <- list(marginal = c(`class 0` = p0, `class 1` = 1 - p0), base = base, criterion = criterion, B1 = B1, B2 = B2, D = D,
+                iteration = iteration, fit.list = fit.list, cutoff = cutoff, subspace = subspace, ranking = rk, scale = scale.parameters)
     class(obj) <- "RaSE"
 
+
+
+
     return(obj)
 }
